package com.hikcreate.bigdata.hdip.source;

import org.mortbay.jetty.Connector;
import org.mortbay.jetty.Server;
import org.mortbay.jetty.nio.SelectChannelConnector;
import org.mortbay.jetty.servlet.ServletHolder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * @author chencong13
 * @Description 通过api的方式来控制服务
 * @Time 2020/8/20 14:43
 */
public class ApiServer {
    public static final Logger logger = LoggerFactory.getLogger(ApiServer.class);
    private OracleSourceService service;
    private volatile Server srv;
    private volatile String host = "127.0.0.1";

    /**
     * 方法描述 用于测试
     * 处理逻辑
     *
     * @param:
     * @author chencong13
     * @Time 2020/10/20 9:55
     **/
    public void testStart() {
        service = new OracleSourceService();
        service.start("C:\\Users\\chencong13\\Desktop\\test\\queryConfig3.json");
    }

    public void start(int port) {
        logger.info("JettySource start...");
        srv = new Server();
        service = new OracleSourceService();
        // Connector Array
        Connector[] connectors = new Connector[1];
        SelectChannelConnector connector = new SelectChannelConnector();
        connector.setReuseAddress(true);
        connectors[0] = connector;
        connectors[0].setHost(host);
        connectors[0].setPort(port);
        srv.setConnectors(connectors);
        try {
            org.mortbay.jetty.servlet.Context root =
                    new org.mortbay.jetty.servlet.Context(srv, "/", org.mortbay.jetty.servlet.Context.SESSIONS);
            root.addServlet(new ServletHolder(new FlumeHTTPServlet()), "/");
            srv.start();
            srv.join();
        } catch (Exception ex) {
            logger.error("Error while starting HTTPSource. Exception follows.", ex);
        }
        logger.info("JettySource start end...");
    }

    /**
     * 方法描述 关闭jetty服务
     * 处理逻辑
     *
     * @param
     * @return
     * @author chencong13
     * @Time 2020/8/20 15:37
     */
    public void shutdown() {
        logger.info("JettySource shutdown...");
        try {
            srv.stop();
            srv.join();
            srv = null;
        } catch (Exception ex) {
            logger.error("Error while stopping HTTPSource. Exception follows.", ex);
        }
    }

    private class FlumeHTTPServlet extends HttpServlet {

        private static final long serialVersionUID = 4891924863218790344L;

        @Override
        public void doPost(HttpServletRequest request, HttpServletResponse response)
                throws IOException {
            String characterEncoding = request.getCharacterEncoding();
            String urlPath = request.getServletPath();
            String msg = "";
            if (urlPath.contains("/querycontrol")) {
                String command = request.getParameter("command");
                System.out.println("command = " + command);
                switch (command) {
                    case "start":
                        String configPath = request.getParameter("configPath");
                        logger.info("configPath is {}", configPath);
                        msg = service.start(configPath);
                        break;
                    case "shutdown":
                        msg = service.shuntDown();
                        break;
                    case "status":
                        msg = service.getStatus();
                        break;
                    case "saveoffset":
                        msg = service.saveOffset();
                        break;
                    case "checkoffset":
                        msg = service.getOffset();
                        break;
                    case "kill":
                        service.shuntDown();
                        shutdown();
                        break;
                    default:
                        break;
                }
            }
            msg = "result:" + msg;
            reponseIt(response, msg, characterEncoding, urlPath);
        }

        @Override
        public void doGet(HttpServletRequest request, HttpServletResponse response)
                throws IOException {
            doPost(request, response);
        }

        private void reponseIt(HttpServletResponse response, String info, String encoding, String header) throws IOException {
            response.setCharacterEncoding(encoding);
            response.setStatus(HttpServletResponse.SC_OK);
            response.getWriter().write(info);
            response.setHeader("header", header);
            response.flushBuffer();
        }
    }

    /**
     * 方法描述 启动类
     * 处理逻辑 需要传服务端口
     * @param: args
     * @author chencong13
     * @Time 2020/10/20 9:56
     **/
    public static void main(String[] args) {
        if (args.length == 0) {
            logger.error("please input your server port!");
            return;
        }
        String port = args[0];
        ApiServer apiServer = new ApiServer();
        apiServer.start(Integer.parseInt(port));
        //apiServer.testStart();
    }
}
