package com.hikcreate.bigdata.hdip.source;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 常用的帮助类
 *
 * @author Administrator
 * @Time 2020-08-20 12:12:12
 */

public class CommonUtils {
    public static final Logger logger = LoggerFactory.getLogger(CommonUtils.class);
    /**
     * 默认 yyyy-MM-dd HH:mm:ss
     */
    static public String df1 = "yyyy-MM-dd HH:mm:ss";
    /**
     * yyyy/MM/dd HH:mm:ss
     */
    static public String df2 = "yyyy/MM/dd HH:mm:ss";
    /**
     * yyyy年MM月dd日 HH时mm分ss秒
     */
    static public String df3 = "yyyy年MM月dd日 HH时mm分ss秒";
    /**
     * 默认 yyyy-MM-dd
     */
    static public String df4 = "yyyy-MM-dd";
    /**
     * yyyy/MM/dd
     */
    static public String df5 = "yyyy/MM/dd";
    /**
     * yyyy年MM月dd日
     */
    static public String df6 = "yyyy年MM月dd日";

    /**
     * yyyy-MM-dd HH:ss:mm.SSS
     */
    static public String df7 = "yyyy-MM-dd HH:ss:mm.SSS";

    /**
     * yyyy/MM/dd HH:ss:mm.SSS
     */
    static public String df8 = "yyyy/MM/dd HH:ss:mm.SSS";
    static public String df9 = "yyyy-MM-dd HH:ss:mm.SSSSSS";

    /**
     * 方法描述 判断是否是日期格式的字符串
     * 处理逻辑 正则匹配
     *
     * @param strDate 日期字符串
     * @return true:符合日期格式
     * @author chencong13
     * @Time 2020/8/20 9:59
     */
    public static boolean isDate(String strDate) {
        Pattern pattern = Pattern
                .compile("^((\\d{2}(([02468][048])|([13579][26]))[\\-\\/\\s]?((((0?[13578])|(1[02]))[\\-\\/\\s]?((0?[1-9])|([1-2][0-9])|(3[01])))|(((0?[469])|(11))[\\-\\/\\s]?((0?[1-9])|([1-2][0-9])|(30)))|(0?2[\\-\\/\\s]?((0?[1-9])|([1-2][0-9])))))|(\\d{2}(([02468][1235679])|([13579][01345789]))[\\-\\/\\s]?((((0?[13578])|(1[02]))[\\-\\/\\s]?((0?[1-9])|([1-2][0-9])|(3[01])))|(((0?[469])|(11))[\\-\\/\\s]?((0?[1-9])|([1-2][0-9])|(30)))|(0?2[\\-\\/\\s]?((0?[1-9])|(1[0-9])|(2[0-8]))))))(\\s(((0?[0-9])|([1-2][0-3]))\\:([0-5]?[0-9])((\\s)|(\\:([0-5]?[0-9])))))?$");
        Matcher m = pattern.matcher(strDate);
        if (m.matches()) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 方法描述 判断是否是日期格式
     * 处理逻辑 规则判断
     *
     * @param strDate:字符串日期
     * @return true:符合日期格式
     * @author chencong13
     * @Time 2020/8/20 10:00
     */
    public static boolean isDate2(String strDate) {
        if (strDate == null || "".equals(strDate)) {
            return false;
        }
        if (strDate.contains(":") && strDate.contains(" ") && strDate.length() >= 10) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 将字符串转化为日期
     *
     * @param date 日期
     * @param df   字符串格式，默认 Sat Mar 23 00:00:00 GMT+08:00
     * @return Date 日期，格式 Sat Mar 23 00:00:00 GMT+08:00
     * @throws ParseException
     */

    /**
     * 方法描述 将字符串转化为日期
     * 处理逻辑
     *
     * @return Date 日期
     * @throws ParseException 解析异常
     * @author chencong13
     * @Time 2020/8/20 10:01
     */
    public static Date stringParseDate(String date, String df) throws ParseException {
        SimpleDateFormat formatter = null;
        if (isBlank(df)) {
            formatter = new SimpleDateFormat(df1);
        } else {
            formatter = new SimpleDateFormat(df);
        }
        Date parseDate = formatter.parse(date);
        return parseDate;
    }

    /**
     * 方法描述 判断字符串是否为空
     * 处理逻辑
     *
     * @param source
     * @return true:为空
     * @author chencong13
     * @Time 2020/8/20 10:03
     */
    public static final boolean isBlank(String source) {
        return (source == null || "".equals(source.trim()));
    }

    /**
     * 方法描述 判断时间字符串的日期格式
     * 处理逻辑 正则匹配
     *
     * @param timeStr:时间字符串
     * @return 日期格式
     * @author chencong13
     * @Time 2020/8/20 10:04
     */
    public static String getDateFormat(String timeStr) {
        if (timeStr.contains(".")) {
            timeStr = timeStr.split("\\.")[0];
        }
        String regex1 = "\\d{4}-\\d{2}-\\d{2}\\s{1}\\d{2}:\\d{2}:\\d{2}";
        String regex2 = "\\d{4}/\\d{2}/\\d{2}\\s{1}\\d{2}:\\d{2}:\\d{2}";
        String regex3 = "\\d{4}年\\d{2}月\\d{2}日\\s{1}\\d{2}时\\d{2}分\\d{2}秒";
        String regex4 = "\\d{4}-\\d{2}-\\d{2}";
        String regex5 = "\\d{4}/\\d{2}/\\d{2}";
        String regex6 = "\\d{4}年\\d{2}月\\d{2}日";
        String regex7 = "\\d{4}-\\d{2}-\\d{2}\\s{1}\\d{2}:\\d{2}:\\d{2}.\\d{3}";
        String regex8 = "\\d{4}/\\d{2}/\\d{2}\\s{1}\\d{2}:\\d{2}:\\d{2}.\\d{3}";
        String regex9 = "\\d{4}-\\d{2}-\\d{2}\\s{1}\\d{2}:\\d{2}:\\d{2}.\\d{6}";

        // 编译正则表达式
        Pattern pattern1 = Pattern.compile(regex1);
        Pattern pattern2 = Pattern.compile(regex2);
        Pattern pattern3 = Pattern.compile(regex3);
        Pattern pattern4 = Pattern.compile(regex4);
        Pattern pattern5 = Pattern.compile(regex5);
        Pattern pattern6 = Pattern.compile(regex6);
        Pattern pattern7 = Pattern.compile(regex7);
        Pattern pattern8 = Pattern.compile(regex8);
        Pattern pattern9 = Pattern.compile(regex9);

        // 忽略大小写的写法
        // Pattern pat = Pattern.compile(regEx, Pattern.CASE_INSENSITIVE);
        Matcher matcher1 = pattern1.matcher(timeStr);
        Matcher matcher2 = pattern2.matcher(timeStr);
        Matcher matcher3 = pattern3.matcher(timeStr);
        Matcher matcher4 = pattern4.matcher(timeStr);
        Matcher matcher5 = pattern5.matcher(timeStr);
        Matcher matcher6 = pattern6.matcher(timeStr);
        Matcher matcher7 = pattern7.matcher(timeStr);
        Matcher matcher8 = pattern8.matcher(timeStr);
        Matcher matcher9 = pattern9.matcher(timeStr);

        if (matcher1.matches()) {
            return df1;
        }
        if (matcher2.matches()) {
            return df2;
        }
        if (matcher3.matches()) {
            return df3;
        }
        if (matcher4.matches()) {
            return df4;
        }
        if (matcher5.matches()) {
            return df5;
        }
        if (matcher6.matches()) {
            return df6;
        }
        if (matcher7.matches()) {
            return df7;
        }
        if (matcher8.matches()) {
            return df8;
        }
        if (matcher9.matches()) {
            return df9;
        }
        return null;
    }

    /**
     * 方法描述 创建文件目录
     * 处理逻辑
     *
     * @param path:目录的路径
     * @return 无
     * @author chencong13
     * @Time 2020/8/20 10:05
     */
    public static void createDir(String path) {
        File file = new File(path);
        if (!file.exists()) {
            file.mkdirs();
        }
    }

    /**
     * 方法描述 得到当前日期
     * 处理逻辑 格式 yyyy年MM月dd日 HH时mm分ss秒
     *
     * @param
     * @return 无
     * @author chencong13
     * @Time 2020/8/20 10:06
     */
    public static String getCurrentDate() {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat(df1);
        return simpleDateFormat.format(new Date());
    }

    /**
     * 方法描述 文件重命名
     * 处理逻辑 将文件one重命名成two
     *
     * @return
     * @author chencong13
     * @Time 2020/8/20 10:07
     */
    public static void renameFile(String one, String two) {
        File s = new File(one);
        File d = new File(two);
        boolean b = s.renameTo(d);
        if (b) {
            s.delete();
        }
    }

    /**
     * 方法描述 清空文件内容
     * 处理逻辑
     *
     * @param
     * @return
     * @throws
     * @author chencong13
     * @Time 2020/8/20 11:39
     */
    public static void clearFileContent(String path) {
        try {
            FileWriter fw = new FileWriter(path);
            fw.write("");
            fw.flush();
            fw.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 方法描述 根据时间进行解析
     * 处理逻辑
     * @param: date
     * @author chencong13
     * @Time 2020/10/20 9:58
     **/
    public static long parseDate(String date) {
        String dateFormat = getDateFormat(date);
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat(dateFormat);
        try {
            Date parse = simpleDateFormat.parse(date);
            return parse.getTime();
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return -1;
    }

    /**
     * 方法描述 根据给定的毫秒数，获得当前时间
     * 处理逻辑
     * @param: time
     * @author chencong13
     * @Time 2020/10/20 9:57
     **/
    public static String getTargetDate(long time) {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat(df1);
        return simpleDateFormat.format(new Date(time));
    }

    public static void main(String[] args) {
        for (int i = 0; i < 1000; i++) {
            logger.info("..a...fsfjlsjflsifhisfjlsfjslfjslfhslfhsnkfnhskfhslfjslf" + i);
        }
    }

    /*
    public static void testIn() {
        while (true) {
            testInsert("A");
            testInsert("B");
            testInsert("C");
            try {
                Thread.sleep(10 * 1000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }
    public static void main(String[] args) {
        //System.out.println(parseDate("2020-09-23 10:09:01")+" "+System.currentTimeMillis());
        //testIn();
        //testInsert("A");
        //checkIt("C:\\\\Users\\\\chencong13\\\\Desktop\\\\test\\\\ftp");
    }

    public static void checkIt(String path) {
        File dir = new File(path);
        File[] files = dir.listFiles();
        Map<String, Integer> result = new HashMap<String, Integer>();
        for (File f : files) {
            String[] split = f.getName().split("#");
            Integer integer = result.get(split[2]);
            if (integer == null) {
                result.put(split[2], checkIt2(f));
            } else {
                result.put(split[2], integer + checkIt2(f));
            }
        }
        for (String k : result.keySet()) {
            System.out.println(k + ":" + result.get(k));
        }
    }

    public static int checkIt2(File f) {
        try {
            BufferedReader br = new BufferedReader(new FileReader(f));
            String line = "";
            int count = 0;
            while ((line = br.readLine()) != null) {
                if (line.length() > 38) {
                    count++;
                }
            }
            br.close();
            return count;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return 0;
    }

    public static void testInsert(String tableName) {
        Connection connection = DruidUtils.getConnection();
        String sql = "insert into test." + tableName + " (A,B,C,D) values (?,?,?,?)";
        PreparedStatement preparedStatement = null;
        try {
            preparedStatement = connection.prepareStatement(sql);
            for (int i = 0; i < 2; i++) {
                Thread.sleep(1000);
                preparedStatement.setString(1, UUID.randomUUID().toString());
                preparedStatement.setTimestamp(2, new Timestamp(System.currentTimeMillis()));
                preparedStatement.setString(3, 101 + "");
                preparedStatement.setString(4, tableName + i);
                preparedStatement.addBatch();
            }
            preparedStatement.executeBatch();
        } catch (SQLException e) {
            e.printStackTrace();
        } catch (InterruptedException e) {
            e.printStackTrace();
        } finally {
            try {
                preparedStatement.close();
                connection.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
    }*/

}
