package com.hikcreate.bigdata.hdip.source;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * @author chencong13
 * @Description 执行器
 * @Time 2020/9/18 10:58
 */
public class Executor implements Runnable {
    private static final Logger logger = LoggerFactory.getLogger(Executor.class.getName());
    private ConcurrentLinkedQueue<QueryTask> tasks; //消息队列
    private boolean flag = true;
    private ExecutorService executors; //线程池

    public Executor(ConcurrentLinkedQueue<QueryTask> tasks) {
        this.tasks = tasks;
        flag = true;
        executors = Executors.newCachedThreadPool();
    }
    /**
     * 方法描述 从消息队列中取出task，然后执行
     * 处理逻辑
     * @param:
     * @author chencong13
     * @Time 2020/10/20 10:03
     **/
    @Override
    public void run() {
        logger.info("start execute");
        while (flag) {
            QueryTask poll = tasks.poll();
            if (poll != null && !poll.isRunning()) {
                logger.info("execute - " + poll.toString() + " task size = " + tasks.size());
                executors.execute(poll);
            }
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 方法描述 停止类
     * 处理逻辑
     * @param:
     * @author chencong13
     * @Time 2020/10/20 10:04
     **/
    public void stop() {
        flag = false;
        executors.shutdownNow();
    }
}
