package com.hikcreate.bigdata.hdip.source;

import com.google.gson.Gson;
import oracle.sql.BLOB;
import oracle.sql.CLOB;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.sql.*;
import java.text.ParseException;
import java.util.*;
import java.util.concurrent.ConcurrentLinkedQueue;

/**
 * 采集Oracle表task，任务
 *
 * @author Administrator
 * @Time 2020-08-20 12:12:12
 */
public class QueryTask implements Runnable {
    /**
     * 日志打印
     */
    private static final Logger log = LoggerFactory.getLogger(QueryTask.class);
    /**
     * 文件未写完
     */
    public static final String NOT_COMPELTE = ".temp";
    /**
     * 文件完成后缀
     */
    public static final String COMPELTE = ".hik";
    public static final String HIKNULL = "fromjdbcsource-null";
    /**
     * 标记：任务出错保存offset
     */
    public static final int OFFSET_ERR = 2;
    /**
     * 标记：任务正常保存offset
     */
    public static final int OFFSET_50 = 1;
    private TableBean tableBean; //采集表的详情
    private String tempPath; //文件的临时保存路径
    private String offset; //偏移量
    private boolean stop; //是否退出的标记
    private boolean isRunning;//是否正在运行的标记
    private String sqlPre; //sql的前缀
    private long no; //文件编号
    private QueryCallback queryCallback; //回调
    private int columnCount;//字段的个数
    private String ftpPath;//ftp的目录
    private String jdbcType;//数据库类型

    /**
     * 方法描述 构造函数
     * 处理逻辑
     *
     * @return
     * @author chencong13
     * @Time 2020/8/20 15:39
     */
    public QueryTask(TableBean tableBean, String tempPath, String offset, long no, String ftpPath, String jdbcType) {
        this.tableBean = tableBean;
        this.tempPath = tempPath;
        stop = false;
        this.offset = offset;
        this.no = no;
        this.columnCount = -1;
        this.ftpPath = ftpPath;
        this.jdbcType = jdbcType;
        this.isRunning = false;
    }

    public TableBean getTableBean() {
        return this.tableBean;
    }

    /**
     * 方法描述 添加监听器对象
     * 处理逻辑
     *
     * @param queryCallback：监听器对象
     * @return
     * @throws
     * @author chencong13
     * @Time 2020/8/20 15:40
     */
    public void addLister(QueryCallback queryCallback) {
        this.queryCallback = queryCallback;
    }

    /**
     * 方法描述 run方法
     * 处理逻辑
     *
     * @param
     * @return
     * @throws
     * @author chencong13
     * @Time 2020/8/20 15:41
     */
    @Override
    public void run() {
        log.info("start query {},offset is {}", tableBean.getDatabase() + "." + tableBean.getTable(), offset);
        String executeSql = genSql();
        String genSql = tableBean.getGenSql();
        if (genSql != null) {
            executeSql = genSql;
        }
        executeSql(executeSql);
    }

    /**
     * 方法描述 生成查询sql
     * 处理逻辑
     *
     * @param
     * @return 生成的sql
     * @throws
     * @author chencong13
     * @Time 2020/8/20 15:42
     */
    private String genSql() {
        StringBuilder sql = new StringBuilder();
        String incrField = tableBean.getIncrField();
        if (sqlPre == null) {
            String database = tableBean.getDatabase();
            String table = tableBean.getTable();
            List<String> fileds = tableBean.getFields();
            sql.append("select ");
            for (String str : fileds) {
                sql.append(str + ",");
            }
            sql.setLength(sql.length() - 1);
            sql.append(" from ");
            sql.append(database + "." + table + " where 1=1 ");
            sqlPre = sql.toString();
            sql.setLength(0);
        }
        sql.append(sqlPre);
        switch (tableBean.getIncrType()) {
            case TableBean.NUM_TYPE:
                sql.append("and " + incrField + ">" + offset);
                break;
            case TableBean.TIME_TYPE:
                if ("oracle".equalsIgnoreCase(jdbcType)) {
                    sql.append("and " + incrField + ">to_timestamp('" + offset + "','yyyy-mm-dd hh24:mi:ss.ff') and " + incrField + "<=sysdate-30/24/60/60");
                } else if ("Mysql_5".equalsIgnoreCase(jdbcType)) {
                    sql.append("and " + incrField + ">'" + offset + "' and " + incrField + "<=DATE_SUB(SYSDATE(),INTERVAL 30 SECOND)");
                } else if ("PostgreSql".equalsIgnoreCase(jdbcType)) {
                    sql.append("and " + incrField + ">'" + offset + "' and " + incrField + "<=(now()::TIMESTAMP+'-30sec')");
                } else {
                    log.info("other sql type");
                }
                break;
            case TableBean.STR_TYPE:
                sql.append("and " + incrField + ">'" + offset + "'");
                break;
            case TableBean.WHOLE_TYPE:
                break;
            default:
                break;
        }
        String executeSql = sql.toString();
        return executeSql;
    }


    /**
     * 方法描述 执行sql逻辑
     * 处理逻辑
     *
     * @param
     * @return
     * @throws
     * @author chencong13
     * @Time 2020/8/20 15:43
     */
    private void executeSql(String executeSql) {
        List<String> tempData = new ArrayList<>();
        ResultSet resultSet = null;
        String tableName = tableBean.getDatabase() + "." + tableBean.getTable();
        PreparedStatement preparedStatement = null;
        Connection connection = DBUtils.getConnection();
        if (connection == null) {
            log.warn("DruidUtils.getConnection() is null");
            return;
        }
        try {
            isRunning = true;
            preparedStatement = connection.prepareStatement(executeSql);
            resultSet = preparedStatement.executeQuery();
            resultSet.setFetchSize(10000);
            String timeSys = String.valueOf(System.currentTimeMillis());
            StringBuffer sb = new StringBuffer();
            boolean goon = true;
            String newField = "";
            while (resultSet.next()) {
                if (stop) {
                    break;
                }
                ResultSetMetaData metaData = resultSet.getMetaData();
                int columnCount = metaData.getColumnCount();
                if (goon) {
                    String incrColum = "";
                    if (this.columnCount == -1) {
                        this.columnCount = columnCount;
                    } else {
                        if (this.columnCount != columnCount) {
                            incrColum = genIncrColum(metaData, columnCount - this.columnCount);
                            this.columnCount = columnCount;
                        }
                    }
                    newField = incrColum;
                    log.info("new field = " + newField);
                    goon = false;
                }

                //对于增量更新增量字段
                if (!TableBean.WHOLE_TYPE.equals(tableBean.getIncrType())) {
                    int incrIndex = resultSet.findColumn(tableBean.getIncrField());
                    String incrStr = resultSet.getString(incrIndex);
                    updateOffset(incrStr);
                }
                for (int i = 1; i <= columnCount; i++) {
                    String columnName = metaData.getColumnName(i);
                    int columnType = metaData.getColumnType(i);
                    String value = "";
                    if (Types.BLOB == columnType) {
                        BLOB imageCol = (BLOB) resultSet.getBlob(columnName);
                        value = handleBlob(imageCol);
                    } else if (Types.CLOB == columnType) {
                        CLOB contentCol = (CLOB) resultSet.getClob(columnName);
                        value = handleClob(contentCol);
                    } else {
                        Object object = resultSet.getObject(resultSet.findColumn(columnName));
                        if (object == null) {
                            value = HIKNULL;
                        } else {
                            value = String.valueOf(object);
                            value = value.replace("\n", ",");
                        }
                    }
                    sb.append(value + "\001");
                }
                sb.setLength(sb.length() - 1);
                tempData.add(sb.toString());
                sb.setLength(0);
                String sinkSize = tableBean.getSinkSize();
                int intSinkSize = 10000;
                if (sinkSize != null) {
                    intSinkSize = Integer.parseInt(sinkSize);
                }
                if (tempData.size() >= intSinkSize) {
                    if (no % 20 == 0) {
                        queryCallback.handlerMsg(QueryTask.OFFSET_50, tableName);
                    }
                    if (!TableBean.WHOLE_TYPE.equals(tableBean.getIncrType())) {
                        log.info("update offset is {}", offset);
                        flushFile(getHeader(newField), tempData, null, null, null);
                    } else {
                        flushFile(getHeader(newField), tempData, "all", null, timeSys);
                    }
                    tempData.clear();
                    if (!CommonUtils.isBlank(newField)) {
                        newField = "";
                    }
                }
            }
            if (tempData.size() != 0) {
                if (!TableBean.WHOLE_TYPE.equals(tableBean.getIncrType())) {
                    log.info("update offset is {}", offset);
                    flushFile(getHeader(newField), tempData, null, null, null);
                } else {
                    flushFile(getHeader(newField), tempData, "all", "end", timeSys);
                }
                log.info("table is {},update offset is {}", tableName, offset);
                tempData.clear();
            }
        } catch (Exception e) {
            log.error(tableName, e);
            stop();
            queryCallback.handlerMsg(2, tableName);
        } finally {
            try {
                isRunning = false;
                log.info("close connetction");
                preparedStatement.close();
                resultSet.close();
                connection.close();
            } catch (SQLException e) {
                log.error(tableName, e);
            }
        }
    }

    /**
     * 方法描述 处理图片字段
     * 处理逻辑 将图片base64转码，然后返回
     * @param: imageCol
     * @author chencong13
     * @Time 2020/10/20 10:06
     **/
    private String handleBlob(BLOB imageCol) {
        InputStream input = null;
        if (imageCol == null) {
            return HIKNULL;
        }
        String result = "";
        try {
            input = imageCol.getBinaryStream();
            int len = (int) imageCol.length();
            if (len == 0) {
                result = "blob-null";
            } else {
                byte[] buffer = new byte[len];
                input.read(buffer);
                result = Base64.getEncoder().encodeToString(buffer);
            }
        } catch (Exception e) {
            log.error("handleBlob error", e);
        } finally {
            try {
                input.close();
            } catch (IOException e) {
                log.error("handleBlob catch error", e);
            }
        }
        return result;
    }

    /**
     * 方法描述 获取运行标记
     * 处理逻辑
     * @param:
     * @author chencong13
     * @Time 2020/10/20 10:06
     **/
    public boolean isRunning() {
        return isRunning;
    }

    /**
     * 方法描述 处理大文本字段
     * 处理逻辑
     * @param: contentCol
     * @author chencong13
     * @Time 2020/10/20 10:07
     **/
    private String handleClob(CLOB contentCol) {
        if (contentCol == null) {
            return HIKNULL;
        }
        InputStream input = null;
        String result = "";
        try {
            input = contentCol.getAsciiStream();
            int len = (int) contentCol.length();
            if (len == 0) {
                result = "clob-null";
            } else {
                byte[] buffer = new byte[len];
                input.read(buffer);
                result = Base64.getEncoder().encodeToString(buffer);
            }
        } catch (Exception e) {
            log.error("handleClob error", e);
        } finally {
            try {
                input.close();
            } catch (IOException e) {
                log.error("handleClob catch error", e);
            }
        }
        return result;
    }

    /**
     * 方法描述 表结构变化，得到新增的字段
     * 处理逻辑
     *
     * @param
     * @return
     * @throws
     * @author chencong13
     * @Time 2020/8/25 14:56
     */
    private String genIncrColum(ResultSetMetaData rd, int tag) throws SQLException {
        if (tag <= 0) {
            return "";
        }
        int count = rd.getColumnCount();
        List<Map<String, Object>> tempList = new ArrayList<>();
        for (int i = tag - 1; i >= 0; i--) {
            Map<String, Object> temp = new HashMap<>();
            int index = count - i;
            int columnType = rd.getColumnType(index);
            int precision = rd.getPrecision(index);
            int scale = rd.getScale(index);
            String columnName = rd.getColumnName(index);
            temp.put("columnType", columnType);
            temp.put("precision", precision);
            temp.put("scale", scale);
            temp.put("columnName", columnName);
            tempList.add(temp);
        }
        Gson gson = new Gson();
        String s = gson.toJson(tempList);
        return s;
    }

    /**
     * 方法描述 停止该任务
     * 处理逻辑
     *
     * @param
     * @return
     * @throws
     * @author chencong13
     * @Time 2020/8/20 15:43
     */
    public void stop() {
        log.info("querytask {} stopped", tableBean.getDatabase() + "." + tableBean.getTable());
        stop = true;
        isRunning = false;
    }

    /**
     * 方法描述 保存到文件中
     * 处理逻辑
     *
     * @param
     * @return
     * @throws
     * @author chencong13
     * @Time 2020/8/20 15:43
     */
    private void flushFile(List<String> header, List<String> datas, String prefix, String suffix, String timesys) {
        String fileName = getFileName(prefix, suffix, timesys);
        CommonUtils.createDir(tempPath);
        String filePath = tempPath + File.separator + fileName + NOT_COMPELTE;
        BufferedWriter fw = null;
        try {
            fw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(filePath), "UTF-8"));
            if (header != null) {
                for (String k : header) {
                    fw.write(k + "\n");
                }
            }
            if (datas != null) {
                for (String data : datas) {
                    fw.write(data + "\n");
                }
            }
            fw.flush();
            no++;
        } catch (Exception e) {
            log.error("flushFile error", e);
        } finally {
            try {
                fw.close();
                changeFile(fileName);
            } catch (IOException e) {
                log.error("flushFile catch error", e);
            }
        }

    }

    /**
     * 方法描述 关闭文件写对象
     * 处理逻辑
     *
     * @param
     * @return
     * @throws
     * @author chencong13
     * @Time 2020/8/20 15:43
     */
    private void changeFile(String fileName) {
        String oldFileName = tempPath + File.separator + fileName + NOT_COMPELTE;
        String targetFile = ftpPath + File.separator + fileName + COMPELTE;
        CommonUtils.renameFile(oldFileName, targetFile);
    }

    /**
     * 方法描述 获得文件名字
     * 处理逻辑
     *
     * @param
     * @return
     * @throws
     * @author chencong13
     * @Time 2020/8/20 15:44
     */
    private String getFileName(String prefix, String suffix, String timesys) {
        if (timesys == null) {
            timesys = String.valueOf(System.currentTimeMillis());
        }
        String result = no + "#" + tableBean.getDatabase() + "#" + tableBean.getTable() + "#" + timesys;
        if (!CommonUtils.isBlank(prefix)) {
            result = prefix + "#" + result;
        }

        if (!CommonUtils.isBlank(suffix)) {
            result = result + "#" + suffix;
        }
        return result;
    }

    /**
     * 方法描述 构建文件头信息
     * 处理逻辑
     *
     * @param
     * @return
     * @throws
     * @author chencong13
     * @Time 2020/8/20 15:44
     */
    private List<String> getHeader(String nf) {
        List<String> header = new ArrayList<>();
        header.add("database:" + tableBean.getDatabase());
        header.add("table:" + tableBean.getTable());
        header.add("offset:" + offset);
        header.add("time:" + CommonUtils.getCurrentDate());
        header.add("new_fields:" + nf);
        header.add("datas:" + "");
        return header;
    }

    /**
     * 方法描述 刷新offset
     * 处理逻辑
     *
     * @param
     * @return
     * @throws
     * @author chencong13
     * @Time 2020/8/20 15:44
     */
    private void updateOffset(String value) throws ParseException {
        if (CommonUtils.isBlank(value)) {
            return;
        }
        switch (tableBean.getIncrType()) {
            case TableBean.NUM_TYPE:
                long newOffset = Long.parseLong(value);
                long oldOffset = Long.parseLong(offset);
                if (newOffset > oldOffset) {
                    offset = value;
                }
                break;
            case TableBean.TIME_TYPE:
                newOffset = CommonUtils.stringParseDate(value, CommonUtils.getDateFormat(value)).getTime();
                if (!CommonUtils.isDate2(value)) {
                    log.info("!CommonUtils.isDate2(value)");
                    break;
                }
                oldOffset = CommonUtils.stringParseDate(offset, CommonUtils.getDateFormat(offset)).getTime();
                if (newOffset > oldOffset) {
                    offset = value;
                }
                break;
            case TableBean.STR_TYPE:
                offset = value.compareTo(offset) > 0 ? value : offset;
        }
    }

    public boolean getStopFlag() {
        return stop;
    }

    /**
     * 方法描述 获取task当前的offset
     * 处理逻辑
     *
     * @param
     * @return
     * @throws
     * @author chencong13
     * @Time 2020/8/20 15:44
     */
    public String getOffset() {
        return offset;
    }

    /**
     * 方法描述 获取文件编号
     * 处理逻辑
     *
     * @param
     * @return
     * @author chencong13
     * @Time 2020/8/21 16:09
     */
    public long getFileNo() {
        return no;
    }

    @Override
    public String toString() {
        return tableBean.getDatabase() + "." + tableBean.getTable() + " offset = " + offset + ",no = " + no + ", is stop " + stop;
    }
}
