package com.hikcreate.bigdata.hdip.source;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Map;
import java.util.concurrent.ConcurrentLinkedQueue;

/**
 * @author chencong13
 * @Description 调度
 * @Time 2020/9/17 18:28
 */
public class Scheduler implements Runnable {
    Logger logger = LoggerFactory.getLogger(Scheduler.class.getName());
    Map<String, QueryTask> datas; //任务集合
    private boolean flag; //停止标记
    private ConcurrentLinkedQueue<QueryTask> queryTasks; //执行对列


    public Scheduler(Map<String, QueryTask> datas, ConcurrentLinkedQueue<QueryTask> queryTasks) {
        this.datas = datas;
        this.queryTasks = queryTasks;
        flag = true;
    }

    /**
     * 方法描述 通过停止一秒，叠加数量，取余来处理调度
     * 处理逻辑
     * @param:
     * @author chencong13
     * @Time 2020/10/20 10:08
     **/
    @Override
    public void run() {
        logger.info("start scheduler...");
        long count = 0;
        while (flag) {
            count++;
            long startTime = System.currentTimeMillis();
            for (QueryTask qt : datas.values()) {
                String queryMin = qt.getTableBean().getQueryMin();
                boolean running = qt.isRunning();
                if (running) {
                    continue;
                }
                if (queryMin.contains(":")) {
                    String tomorrow = CommonUtils.getTargetDate(System.currentTimeMillis() + 24 * 60 * 60 * 1000L);
                    String tday = tomorrow.split(" ")[0];
                    long targetDate = CommonUtils.parseDate(tday + " " + queryMin.split(" ")[1]);
                    if (targetDate == -1) {
                        logger.warn("parse error,CommonUtils.parseDate");
                    } else {
                        long lag = Math.abs(targetDate - startTime);
                        if (lag < 60 * 1000) {
                            qt.getTableBean().setQueryMin(CommonUtils.getTargetDate(targetDate + 24 * 60 * 60 * 1000L));
                            queryTasks.add(qt);
                            logger.info("scheduler = " + qt.toString() + " cur day " + CommonUtils.getCurrentDate());
                        }
                    }
                } else {
                    int queryMinInt = Integer.parseInt(queryMin);
                    if (count % (queryMinInt * 60) == 0) {
                        logger.info("scheduler = " + qt.toString() + "  count = " + count + " cur day " + CommonUtils.getCurrentDate());
                        queryTasks.add(qt);
                    }
                }
            }
            long endTime = System.currentTimeMillis();
            long sleepTime = 1000 - (endTime - startTime);
            if (sleepTime < 0) {
                logger.info("count-- = " + count + ",datas size = " + datas.size() + ",sleeptime = " + sleepTime);
                sleepTime = 1000;
            }
            try {
                Thread.sleep(sleepTime);
            } catch (InterruptedException e) {
                logger.error("InterruptedException e", e);
            }
        }
    }

    public void stop() {
        flag = false;
    }

}
